//
//  TChunk.cs
//  Copyright 2005 Takeshi Yoneki / OSTRA / OSTRACISM CO., All rights reserved.
//
using System;
using System.Text;
using System.IO;
using System.Net;

namespace GGG4W
{
	/// <summary>
	/// TChunk `NNX
	/// </summary>
	public class TChunk
	{
		public const string CKID_HEAD = "GGHD";
		public const string CKID_PATH = "GGPT";
		public const string CKID_THUMB = "GGTM";
		
		public string m_ckID = "----";
		public int m_ckSize = 0;
		public byte[] m_data = null;

		public TChunk()
		{
			// 
			// TODO: RXgN^ WbNɒǉĂB
			//
		}

		/*
		 * ǂݍ
		 */
		public virtual int Read(ref BinaryReader br) {
			if (br == null)
				return 0;
			byte[] buff = new byte[4];
			if (br.Read(buff, 0 ,4) != 4)
				return 0;
			UTF8Encoding uni = new UTF8Encoding();
			m_ckID = uni.GetString(buff);
			m_ckSize = IPAddress.NetworkToHostOrder(br.ReadInt32());
			if (m_ckSize == 0)
				return 8;
			m_data = new byte[m_ckSize];
			if (br.Read(m_data, 0 , m_ckSize) != m_ckSize)
				return 0;
			return 8 + m_ckSize;
		}

		/*
		 * o
		 */
		public virtual int Write(ref BinaryWriter bw) {
			if (m_data == null)
				return 0;
			if (bw == null)
				return 0;
			UTF8Encoding utf8 = new UTF8Encoding();
			byte[] id = utf8.GetBytes(m_ckID);
			bw.Write(id, 0, 4);
			m_ckSize = m_data.Length;
			bw.Write(IPAddress.HostToNetworkOrder(m_ckSize));
			bw.Write(m_data);
			return 8 + m_ckSize;
		}
	}

	/// <summary>
	/// TChunkHead `Nwb_NX
	/// </summary>
	public class TChunkHead : TChunk {
		public int m_version = 0;
		public int m_pixelNumber = 0;

		public TChunkHead() {
			m_ckID = CKID_HEAD;
			m_version = 0x10000;
		}

		/*
		 * ǂݍݍς݃`N̕ϊ
		 */
		public TChunkHead(ref TChunk ck) {
			m_ckID = ck.m_ckID;
			m_ckSize = ck.m_ckSize;
			m_data = ck.m_data;

			MemoryStream ms = new MemoryStream(m_data);
			BinaryReader msbr = new BinaryReader(ms);
			m_version = IPAddress.NetworkToHostOrder(msbr.ReadInt32());
			if (m_version >= 0x10000) {
				m_pixelNumber = IPAddress.NetworkToHostOrder(msbr.ReadInt32());
			}
			msbr.Close();
			ms.Close();
		}

		/*
		 * o
		 */
		public override int Write(ref BinaryWriter bw) {
			m_data = new byte[256];
			MemoryStream ms = new MemoryStream(m_data);
			BinaryWriter msbw = new BinaryWriter(ms);
			msbw.Write(IPAddress.HostToNetworkOrder(m_version));
			msbw.Write(IPAddress.HostToNetworkOrder(m_pixelNumber));
			msbw.Close();
			ms.Close();
			return base.Write(ref bw);
		}
	}

	/// <summary>
	/// TChunkPath `NpXNX
	/// </summary>
	public class TChunkPath : TChunk {
		public string m_path;

		public TChunkPath() {
			m_ckID = CKID_PATH;
		}

		/*
		 * ǂݍݍς݃`N̕ϊ
		 */
		public TChunkPath(ref TChunk ck) {
			m_ckID = ck.m_ckID;
			m_ckSize = ck.m_ckSize;
			m_data = ck.m_data;

			UTF8Encoding uni = new UTF8Encoding();
			m_path = uni.GetString(m_data);		
		}

		/*
		 * o
		 */
		public override int Write(ref BinaryWriter bw) {
			UTF8Encoding utf8 = new UTF8Encoding();
			m_data = utf8.GetBytes(m_path);
			return base.Write(ref bw);
		}
	}

	/// <summary>
	/// TChunkThumb `NTl[NX
	/// </summary>
	public class TChunkThumb : TChunk {
		public short[] m_thumbnail = null;

		public TChunkThumb() {
			m_ckID = CKID_THUMB;
		}

		/*
		 * ǂݍݍς݃`N̕ϊ
		 */
		public TChunkThumb(ref TChunk ck) {
			m_ckID = ck.m_ckID;
			m_ckSize = ck.m_ckSize;
			m_data = ck.m_data;

			MemoryStream ms = new MemoryStream(m_data);
			BinaryReader msbr = new BinaryReader(ms);
			int count = m_data.Length / 2;
			m_thumbnail = new short[count];
			for (int i = 0; i < count; i++) {
				m_thumbnail[i] = IPAddress.NetworkToHostOrder(msbr.ReadInt16());
			}
			msbr.Close();
			ms.Close();
		}

		/*
		 * o
		 */
		public override int Write(ref BinaryWriter bw) {
			if (m_thumbnail == null || m_thumbnail.Length == 0)
				return 0;
			m_data = new byte[m_thumbnail.Length * 2];
			MemoryStream ms = new MemoryStream(m_data);
			BinaryWriter msbw = new BinaryWriter(ms);
			int count = m_thumbnail.Length;
			for (int i = 0; i < count; i++) {
				msbw.Write(IPAddress.HostToNetworkOrder(m_thumbnail[i]));
			}
			msbw.Close();
			ms.Close();
			return base.Write(ref bw);
		}
	}


}
